/*	$NetBSD$	*/

/*-
 * Copyright (c) 2007 The NetBSD Foundation, Inc.
 * All rights reserved.
 *
 * This code is derived from software contributed to The NetBSD Foundation
 * by ITOH Yasufumi (itohy@NetBSD.org).
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *        This product includes software developed by the NetBSD
 *        Foundation, Inc. and its contributors.
 * 4. Neither the name of The NetBSD Foundation nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE NETBSD FOUNDATION, INC. AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FOUNDATION OR CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * USB memory allocation.
 * Currently we use malloc(9) directly.  If memory fragmentation or
 * overhead matters, it should be rewritten using chunk allocations.
 */

#include <sys/cdefs.h>
__KERNEL_RCSID(0, "$NetBSD$");

#include <sys/param.h>
#include <sys/systm.h>
#include <sys/kernel.h>
#include <sys/malloc.h>
#if defined(__NetBSD__) || defined(__OpenBSD__)
#include <sys/device.h>		/* for usbdivar.h */
#elif defined(__FreeBSD__)
#include <sys/endian.h>
#include <sys/module.h>
#endif
#include <sys/queue.h>

#include <machine/endian.h>

#ifdef __NetBSD__
#include <sys/extent.h>
#include <uvm/uvm_extern.h>
#endif

#include <dev/usb/usb.h>
#include <dev/usb/usbdi.h>
#include <dev/usb/usbdivar.h>
#include <dev/usb/usb_mem_nodma.h>

#ifdef USB_DEBUG
#define DPRINTF(x)	if (usbdebug) logprintf x
#define DPRINTFN(n,x)	if (usbdebug>(n)) logprintf x
extern int usbdebug;
#else
#define DPRINTF(x)
#define DPRINTFN(n,x)
#endif

Static void		usb_mem_free_buffers(usb_mem_tag_t *);

Static void
usb_mem_free_buffers(usb_mem_tag_t *utag)
{
	struct usb_buffer_mem *ub, *ub1;
	int s;

	s = splusb();
	if ((ub = SIMPLEQ_FIRST(&utag->ubs_tobefreed)) != NULL)
		SIMPLEQ_INIT(&utag->ubs_tobefreed);
	splx(s);
	for ( ; ub; ub = ub1) {
		ub1 = SIMPLEQ_NEXT(ub, ub_next);
		free(ub, M_USB);
		utag->nbufs--;
	}
}

usbd_status
usb_alloc_buffer_mem(usb_mem_tag_t *utag, void *buf, size_t size,
	struct usb_buffer_mem **pub,
	void **pbufadr)
{

	if (!SIMPLEQ_EMPTY(&utag->ubs_tobefreed))
		usb_mem_free_buffers(utag);

	if (buf == NULL && size != 0) {
		/*
		 * allocate buffer
		 */
		if (size < sizeof(struct usb_buffer_mem))
		    size = sizeof(struct usb_buffer_mem);
		if ((*pub = *pbufadr = malloc(size, M_USB, M_WAITOK)) == NULL)
			return (USBD_NOMEM);
		DPRINTFN(5, ("usb_alloc_buffer_mem: ub=%p size=%d\n",
			     *pub, size));
		utag->nbufs++;
	} else {
		/*
		 * do nothing --- just use the buffer
		 */
		*pbufadr = buf;
		*pub = NULL;
		DPRINTFN(5, ("usb_alloc_buffer_mem: no alloc\n"));
	}

	return (USBD_NORMAL_COMPLETION);
}

void
usb_free_buffer_mem(usb_mem_tag_t *utag, struct usb_buffer_mem *ub,
	enum usbd_waitflg waitflg)
{
	int s;

	DPRINTFN(5, ("usb_free_buffer_mem: ub=%p\n", ub));

	if (waitflg == U_WAITOK && !SIMPLEQ_EMPTY(&utag->ubs_tobefreed))
		usb_mem_free_buffers(utag);

	if (ub) {
		if (waitflg == U_WAITOK) {
			free(ub, M_USB);
			utag->nbufs--;
		} else {
			s = splusb();
			SIMPLEQ_INSERT_TAIL(&utag->ubs_tobefreed, ub, ub_next);
			splx(s);
			/* XXX request free */
		}
	}
}

void
usb_mem_tag_init(usb_mem_tag_t *utag)
{

	SIMPLEQ_INIT(&utag->ubs_tobefreed);
	utag->nbufs = 0;
	DPRINTFN(5, ("usb_mem_tag_init: utag=%p\n", utag));
}

void
usb_mem_tag_finish(usb_mem_tag_t *utag)
{

	/* current usage */
#if 1
	printf("usb_mem_tag_finish: utag=%p, %d bufs\n", utag, utag->nbufs);
#else
	DPRINTF(("usb_mem_tag_finish: utag=%p, %d bufs\n", utag, utag->nbufs));
#endif

	/* free buffers */
	usb_mem_free_buffers(utag);

#if 1	/* DIAGNOSTIC */
	if (utag->nbufs)
		printf("usb_dma_tag_finish: leaked %d bufs\n", utag->nbufs);
#endif
}
